// This is the LCD Driver for the iPod External Display board
// Project: Pod External Display board
// Comment: This code is used to drive the LCD in 4-bit mode
//
// Author:	Andy Beck
// Revs:	01- Initial Release

#include <pic.h>
#include "LCD_Driver.h"

#define TOGGLE_E_LINE  {E_PIN = 1; 	asm("nop"); E_PIN = 0;}		// Toggles the display's 'E' pin

// Initialize the LCD
void lcd_init(void)
{
 
    TRIS_RW = 0;    // All control signals made outputs
    TRIS_RS = 0;
    TRIS_E = 0;
    RW_PIN = 0;    // R/W pin made low
    RS_PIN = 0;    // Register select pin made low
    E_PIN = 0;    // Clock pin made low
   	TRIS_DATA_PORT &= 0x0f;	// Make data port output

    // Delay for 15ms to allow for LCD Power on reset
    DelayMs(15);

    DATA_PORT &= 0x0f;
    DATA_PORT |= 0b00100000;    // Function set cmd(4-bit interface)

   	TOGGLE_E_LINE                      // Clock the cmd in

    // Delay for at least 4.1ms
    DelayMs(5);

	DATA_PORT &= 0x0f;              // Function set cmd(4-bit interface)
	DATA_PORT |= 0b00100000;
	TOGGLE_E_LINE                      // Clock the cmd in
	
	// Delay for at least 100us
	DelayMs(1);

    DATA_PORT &= 0x0f;      // Function set cmd(4-bit interface)
    DATA_PORT |= 0b00100000;

   TOGGLE_E_LINE          // Clock cmd in

    // Set data interface width, # lines, font
    LCDCommand(LCD_TYPE);      	// Function set cmd

    // Set DD Ram address to 0
    LCDCommand(LCD_TYPE);

    LCDCommand(DOFF&LCD_DISPLAY_SETUP);

    LCDCommand(DON&LCD_DISPLAY_SETUP);

    LCDCommand(0x01);     		// Clear display

    LCDCommand(SHIFT_CUR_LEFT); // Entry Mode

    LCDCommand(0x80);			// Set DD Ram address to 
}

/*********************************************************************
 * Function:        void LCDCommand(unsigned char cmd)
 *
 * PreCondition:    LCDIsBusy() == FALSE if !defined(LCD_IS_BLOCKING)
 *
 * Input:           cmd - Command to be set to LCD.
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        None
 *
 * Note:            None
 ********************************************************************/
void LCDCommand(unsigned char cmd)
{
    while(LCDIsBusy());      // Wait if LCD busy

    TRIS_DATA_PORT &= 0x0f;	// Make data port output

    DATA_PORT &= 0x0f;		// Send low niblle of command
    DATA_PORT |= cmd&0xf0;
    RW_PIN = 0;         	// Set control signals for command
    RS_PIN = 0;
    ////Delay50Us(1);
   	TOGGLE_E_LINE          	// Clock command in
    
    DATA_PORT &= 0x0f;		// Send high nibble of command	
    DATA_PORT |= (cmd<<4)&0xf0;
    //Delay50Us(1);
   	TOGGLE_E_LINE          	// Clock command in
}

/*********************************************************************
 * Function:        char LCDIsBusy(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          non-zero if LCD controller is ready to accept new
 *                      data or command
 *                  zero otherwise.
 *
 * Side Effects:    None
 *
 * Overview:        None
 *
 * Note:            None
 ********************************************************************/
char LCDIsBusy(void)
{
	char status = 0;
	
    TRIS_DATA_PORT |= 0xf0;	// Make data port input	
    RW_PIN = 1;         // Set the control bits for read
    RS_PIN = 0;
    //Delay50Us(1);

   	E_PIN = 1;          // Clock in the status
    //Delay50Us(1);
    if(DATA_PORT&0x80)
        status =  1;
    E_PIN = 0;      	// Reset clock line

   	TOGGLE_E_LINE      	// Clock out other nibble
    RW_PIN = 0;     	// Reset control line
    
    return(status);       // Return status
}


void LCDPutString(char *string)
{
    char v;

    while( v = *string )
    {
        LCDPut(v);
        string++;
    }
}

void LCDPutROMString(const char *string)
{
    char v;

    while( v = *string )
    {
        LCDPut(v);
        string++;
    }
}


/*********************************************************************
 * Function:        void LCDPut(char data)
 *
 * PreCondition:    LCDInit() is already called AND
 *                  (LCDIsBusy() == FALSE AND !defined(LCD_IS_BLOCKING)
 *
 * Input:           data    - Data to be written
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        None
 *
 * Note:            Data is written to character generator RAM or
 *                  display data RAM depending on how the access is
 *                  setup.
 ********************************************************************/
void LCDPut(char data)
{
    while(LCDIsBusy());      // Wait if LCD busy

    TRIS_DATA_PORT &= 0x0f;		// Make data port output
    
    DATA_PORT &= 0x0f;
    DATA_PORT |= data&0xf0;
    RS_PIN = 1;         // Set control bits
    RW_PIN = 0;
    //Delay50Us(1);
   	TOGGLE_E_LINE          // Clock nibble into LCD
   	
    DATA_PORT &= 0x0f;
    DATA_PORT |= ((data<<4)&0xf0);
    //Delay50Us(1);
   	TOGGLE_E_LINE          // Clock nibble into LCD
}

